@@ -1,4 +1,5 @@
 #include "font_matching.h"
+#include "Rinternals.h"
 #include "types.h"
 #include "caches.h"
 #include "utils.h"
@@ -6,10 +7,13 @@
 #include "font_registry.h"
 #include "font_local.h"
 
+#include <cmath>
 #include <string>
+#include <cstring>
 #include <memory>
 
 #include <cpp11/integers.hpp>
+#include <cpp11/doubles.hpp>
 #include <cpp11/strings.hpp>
 #include <cpp11/function.hpp>
 #include <cpp11/r_string.hpp>
@@ -22,6 +26,8 @@ using strings_t = cpp11::strings;
 using strings_w = cpp11::writable::strings;
 using integers_t = cpp11::integers;
 using integers_w = cpp11::writable::integers;
+using doubles_t = cpp11::doubles;
+using doubles_w = cpp11::writable::doubles;
 using logicals_t = cpp11::logicals;
 using logicals_w = cpp11::writable::logicals;
 
@@ -33,7 +39,7 @@ ResultSet *findFonts(FontDescriptor *);
 FontDescriptor *findFont(FontDescriptor *);
 void resetFontCache();
 
-int locate_systemfont(const char *family, int italic, int weight, int width, char *path, int max_path_length) {
+void locate_systemfont(const char *family, int italic, int weight, int width, FontSettings2& res) {
   const char* resolved_family = family;
   if (strcmp_no_case(family, "") || strcmp_no_case(family, "sans")) {
     resolved_family = SANS;
@@ -54,66 +60,129 @@ int locate_systemfont(const char *family, int italic, int weight, int width, cha
   key.italic = italic;
   FontMap::iterator font_it = font_map.find(key);
   if (font_it != font_map.end()) {
-    strncpy(path, font_it->second.file.c_str(), max_path_length);
-    path[max_path_length] = '\0';
-    return font_it->second.index;
+    strncpy(res.file, font_it->second.file.c_str(), PATH_MAX);
+    res.file[PATH_MAX] = '\0';
+    res.index = font_it->second.index;
+    res.axes = font_it->second.axes.data();
+    res.coords = font_it->second.coords.data();
+    res.n_axes = font_it->second.axes.size();
+    return;
   }
 
-  FontDescriptor font_desc(resolved_family, italic, FontWeight(weight), FontWidth(width));
+  FontDescriptor font_desc(resolved_family, fixed_to_italic(italic), fixed_to_weight(weight), fixed_to_width(width));
   std::unique_ptr<FontDescriptor> font_loc(match_local_fonts(&font_desc));
   if (!font_loc) {
     font_loc = std::unique_ptr<FontDescriptor>(findFont(&font_desc));
   }
 
-  int index = 0;
+  FontLoc cached_loc;
 
   if (!font_loc) {
     list_t fallback = cpp11::as_cpp<list_t>(cpp11::package("systemfonts")["get_fallback"]());
-    strncpy(path, CHAR(STRING_ELT(fallback[0], 0)), max_path_length);
-    index = INTEGER(fallback[1])[0];
+    cached_loc.file = std::string(Rf_translateCharUTF8(STRING_ELT(fallback[0], 0)));
+    cached_loc.index = INTEGER(fallback[1])[0];
   } else {
-    strncpy(path, font_loc->path, max_path_length);
-    index = font_loc->index;
+    cached_loc.file = std::string(font_loc->path);
+    cached_loc.index = font_loc->index;
+    if (font_loc->var_ital) {
+      cached_loc.axes.push_back(ITAL_TAG);
+      cached_loc.coords.push_back(italic);
+    }
+    if (font_loc->var_wght) {
+      cached_loc.axes.push_back(WGHT_TAG);
+      cached_loc.coords.push_back(weight);
+    }
+    if (font_loc->var_wdth) {
+      cached_loc.axes.push_back(WDTH_TAG);
+      cached_loc.coords.push_back(width);
+    }
   }
-  path[max_path_length] = '\0';
+  font_map[key] = cached_loc;
 
-  font_map[key] = {std::string(path), (unsigned int) index};
-
-  return index;
+  strncpy(res.file, cached_loc.file.c_str(), PATH_MAX);
+  res.file[PATH_MAX] = '\0';
+  res.index = cached_loc.index;
+  res.axes = cached_loc.axes.data();
+  res.coords = cached_loc.coords.data();
+  res.n_axes = cached_loc.axes.size();
 }
 
 int locate_font(const char *family, int italic, int bold, char *path, int max_path_length) {
+  FontSettings2 match;
+
   BEGIN_CPP
 
-  FontSettings registry_match;
-  if (locate_in_registry(family, italic, bold, registry_match)) {
-    strncpy(path, registry_match.file, max_path_length);
-    return registry_match.index;
+  if (!locate_in_registry(family, italic, bold, match)) {
+    locate_systemfont(
+      family,
+      italic_to_fixed(italic),
+      weight_to_fixed(bold ? FontWeightBold : FontWeightNormal),
+      0,
+      match
+    );
   }
 
-  return locate_systemfont(family, italic, bold ? FontWeightBold : FontWeightNormal, FontWidthUndefined, path, max_path_length);
-
   END_CPP
 
-  return 0;
+  strncpy(path, match.file, max_path_length);
+  return match.index;
 }
 
 FontSettings locate_font_with_features(const char *family, int italic, int bold) {
-  FontSettings registry_match = {};
-  registry_match.features = NULL;
-  registry_match.n_features = 0;
+  FontSettings2 match = {};
 
   BEGIN_CPP
 
-  if (locate_in_registry(family, italic, bold, registry_match)) {
-    return registry_match;
+  if (!locate_in_registry(family, italic, bold, match)) {
+    locate_systemfont(
+      family,
+      italic_to_fixed(italic),
+      weight_to_fixed(bold ? FontWeightBold : FontWeightNormal),
+      0,
+      match
+    );
   }
-  registry_match.index = locate_systemfont(family, italic, bold ? FontWeightBold : FontWeightNormal, FontWidthUndefined, registry_match.file, PATH_MAX);
 
   END_CPP
 
-  registry_match.file[PATH_MAX] = '\0';
-  return registry_match;
+  return match;
+}
+
+FontSettings2 locate_font_with_features2(const char *family, double italic, double weight, double width, const int* axes, const int* coords, int n_axes) {
+  FontSettings2 match = {};
+
+  BEGIN_CPP
+
+  int italic_int = italic_to_fixed(italic);
+  int weight_int = weight_to_fixed(FontWeight(weight));
+  int width_int = width_to_fixed(FontWidth(width));
+
+
+  for (int i = 0; i < n_axes; ++i) {
+    if (axes[i] == ITAL_TAG) {
+      italic_int = coords[i];
+    } else if (axes[i] == WGHT_TAG) {
+      weight_int = coords[i];
+    } else if (axes[i] == WDTH_TAG) {
+      width_int = coords[i];
+    }
+  }
+
+  int weight2 = fixed_to_weight(weight_int);
+
+  if (!locate_in_registry(family, fixed_to_italic(italic_int), weight2 >=650 && weight2 < 750, match)) {
+    locate_systemfont(
+      family,
+      italic_int,
+      weight_int,
+      width_int,
+      match
+    );
+  }
+
+  END_CPP
+
+  return match;
 }
 
 list_t match_font_c(strings_t family, logicals_t italic, logicals_t bold) {
@@ -147,41 +216,57 @@ list_t match_font_c(strings_t family, logicals_t italic, logicals_t bold) {
   });
 }
 
-data_frame_w locate_fonts_c(strings_t family, logicals_t italic,
-                            integers_t weight, integers_t width) {
+data_frame_w locate_fonts_c(strings_t family, doubles_t italic,
+                            doubles_t weight, doubles_t width) {
   strings_w paths;
   integers_w indices;
   list_w features;
+  list_w variations;
 
-  char file[PATH_MAX + 1];
+  FontSettings2 match = {};
 
   for (R_xlen_t i = 0; i < family.size(); ++i) {
-    if ((width[i] == FontWidthUndefined || width[i] == FontWidthNormal) && (weight[i] == FontWeightNormal || weight[i] == FontWeightBold || weight[i] == FontWeightUndefined)) {
-      list_t match = match_font_c(
-        strings_t(Rf_ScalarString(family[i])),
-        logicals_t(Rf_ScalarLogical(italic[i])),
-        logicals_t(Rf_ScalarLogical(weight[i] != FontWeightNormal))
+    const char* fam = Rf_translateCharUTF8(family[i]);
+    bool standard_width = (width[i] == FontWidthUndefined || width[i] == FontWidthNormal);
+    bool standard_weight = (weight[i] == FontWeightNormal || weight[i] == FontWeightBold || weight[i] == FontWeightUndefined);
+    if (!(standard_width && standard_weight && locate_in_registry(fam, italic[i], weight[i] != FontWeightNormal, match))) {
+      locate_systemfont(
+        fam,
+        italic_to_fixed(italic[i]),
+        weight_to_fixed(weight[i]),
+        width_to_fixed(width[i]),
+        match
       );
-      paths.push_back(cpp11::as_cpp<strings_t>(match[0])[0]);
-      indices.push_back(cpp11::as_cpp<integers_t>(match[1])[0]);
-      integers_t feat = cpp11::as_cpp<integers_t>(match[2]);
-      list_w f_feat({feat.names(), feat});
-      f_feat.attr("class") = {"font_feature"};
-      features.push_back(f_feat);
-    } else {
-      int index = locate_systemfont(Rf_translateCharUTF8(family[i]), italic[i], weight[i], width[i], file, PATH_MAX);
-      paths.push_back(file);
-      indices.push_back(index);
-      list_w f_feat({strings_w(), integers_w()});
-      f_feat.attr("class") = {"font_feature"};
-      features.push_back(f_feat);
     }
+    paths.push_back(match.file);
+    indices.push_back(match.index);
+    strings_w tags(match.n_features);
+    integers_w vals(match.n_features);
+    for (int j = 0; j < match.n_features; ++j) {
+      tags[j] = cpp11::r_string({
+        match.features[j].feature[0],
+        match.features[j].feature[1],
+        match.features[j].feature[2],
+        match.features[j].feature[3]
+      });
+      vals[j] = match.features[j].setting;
+    }
+    list_w f_feat({tags, vals});
+    f_feat.attr("class") = {"font_feature"};
+    features.push_back(f_feat);
+    list_w f_vars({
+      "axis"_nm = integers_w(match.axes, match.axes + match.n_axes),
+      "value"_nm = integers_w(match.coords, match.coords + match.n_axes)
+    });
+    f_vars.attr("class") = {"font_variation"};
+    variations.push_back(f_vars);
   }
 
   data_frame_w res({
     "path"_nm = paths,
     "index"_nm = indices,
-    "features"_nm = features
+    "features"_nm = features,
+    "variations"_nm = variations
   });
   res.attr("class") = {"tbl_df", "tbl", "data.frame"};
 
@@ -228,6 +313,7 @@ data_frame_w system_fonts_c() {
   };
   logicals_w italic(n);
   logicals_w monospace(n);
+  logicals_w variable(n);
 
   int i = 0;
 
@@ -247,9 +333,10 @@ data_frame_w system_fonts_c() {
     }
     italic[i] = (Rboolean) (*it)->italic;
     monospace[i] = (Rboolean) (*it)->monospace;
+    variable[i] = (Rboolean) (*it)->variable;
     ++i;
   }
-  
+
   // Remove local fonts so they don't get deleted at cleanup
   all_fonts->erase(all_fonts->begin(), all_fonts->begin() + get_local_font_list().size());
 
@@ -262,7 +349,8 @@ data_frame_w system_fonts_c() {
     "weight"_nm = weight,
     "width"_nm = width,
     "italic"_nm = italic,
-    "monospace"_nm = monospace
+    "monospace"_nm = monospace,
+    "variable"_nm = variable
   });
   res.attr("class") = {"tbl_df", "tbl", "data.frame"};
   return res;
@@ -276,4 +364,5 @@ void reset_font_cache_c() {
 void export_font_matching(DllInfo* dll) {
   R_RegisterCCallable("systemfonts", "locate_font", (DL_FUNC)locate_font);
   R_RegisterCCallable("systemfonts", "locate_font_with_features", (DL_FUNC)locate_font_with_features);
+  R_RegisterCCallable("systemfonts", "locate_font_with_features2", (DL_FUNC)locate_font_with_features2);
 }
